
#ifndef INC_UART_RX_H_
#define INC_UART_RX_H_

#include "stm32l4xx_hal.h"
#include "command.h"

// UART受信バッファサイズ
#define UART_RX_BUFFER_SIZE  (128)

// UART受信ステート -------------------------------
// 	uart_rx_init() 未初期化
#define UART_RX_STATE_INVALID		(0)
//		コマンド文字のヘッダ('+' or '-') が未検出 (受信文字は無視)
#define UART_RX_STATE_EMPTY			(1)
//		コマンド文字のヘッダを受領済で、その後のコマンド文字列を受領している途中
#define UART_RX_STATE_RECEIVING	(2)
//		コマンド文字の終端文字を受領し、コマンド確定状態で待ち (以降の受信文字は無視)
#define UART_RX_STATE_COMPLETE	(3)
//		コマンド文字列のバッファを超過している (以降の受信文字は無視、但しコマンド終端文字を受けると、UART_RX_STATE_EMPTY に戻される)
#define UART_RX_STATE_OVERFLOW	(4)
//		このステート中は、別途指定のメモリに直接受信バイナリを書込む、別途指定のバイナリバイト数を書込んだら、UART_RX_STATE_EMPTY に戻される
#define UART_RX_STATE_DIRECT			(5)
//		このステート中は、受信文字を無視
#define UART_RX_STATE_LOCK		  	(6)


void uart_rx_init(UART_HandleTypeDef *huart);
uint8_t uart_rx_get_state(void);
uint8_t uart_rx_get_command(uint8_t *command_str, uint8_t max_size);
void uart_rx_reboot(void);
void uart_rx_set_empty(void);
void uart_rx_set_lock(void);
void uart_rx_set_unlock(void);
void uart_rx_set_direct(uint8_t *direct_ptr, uint16_t direct_size);

#endif /* INC_UART_RX_H_ */
